PROCEDURE faterror
*******************************************************************************************
*
* PROCEDURE:	faterror
*
* PROCEDURE LIBRARY: RETPROC.PRG
*
* CALLED FROM:	SQLWRITE
*				sqlwritelong
*				startup.prg
*
* INPUTS: 
*     	cmess 		- error message
*		cmess1		- line with error in it
*		cprogname	- name of program with error in it.
*		nerrorcode 	- error code of error
*		nlineno		- line no of line with error in it
*
* PROCESS:
*		calls faterror
*		Uses both Foxpro error numbers and use defined errors (beginning with the number 3000)
*		Accesses tblmess1 and  global variable Glfromstartup.
*		Modifies global variables gndispcode and gnerror.
*		When this procedure is called from STARTUP.PRG, THISFORMSET and all related global 
*		variables are not accessable.
* 		This includes the prefered language, therefore the message boxes are bilingual.
*
* OUTPUTS:
* 		same as inputs(used to pass parameters to fatal	error form).
*		sysDump.txt		- dump of variables and arrays at time of error
*
* MAINTENANCE DONE:
*	May 2000
*	Grouped error code 1705 with error codes 1569 and 1881 - code is identical.
*	Not sure why 1705 was separated from the other two codes.
*	All 3 error codes seem to deal with the situation where someone is trying to run another
*	instance of the application.
*
*	April 2001/October 2001
*	Error code 1705 (file access denied) can now be called under 3 separate circumstances:
*	1) Another instance of SLIDRET is already running (gcDataTable = 'startup').
*	2) A data table needed by the query is locked by another user, and inaccessible to 
*	   anyone else.  Or there could be permission problems with the data.  In either case, 
*	   the clients would like to see the name of the table printed out with the error message.
*	   (gcDataTable = data table name)
*	3) For completeness, we also include a general check for system tables and query tables.
*	   In this case, we issue a standard system error "File access is denied", because the 
*	   table is not specifically identified.
*	   (gcDataTable = 'query table/system table')
*	So error code 1705, which was grouped with error codes 1569 and 1881 (see above May 2000),
*	has been separated from the other 2 codes again.
*	UPDATE:	Now check for error code 3 (File is in use), grouped with error code 1705.
*
*	April 2001
*	Parameter nerrcode, which contains the error code returned by function ERROR(), 
*	incorrectly returns a zero for error code 3 (File is in use)!
*	To get the correct error code, now use function AERROR to create an array holding info
*	about last VFP error.  The 1st element of the array holds the error code.
*
*	September 2001
*	No need for 2 sets of coding for trapping error codes.
*	Originally, there was one set for production (SLIDRET run via startup.prg), and another 
*	one for SDD testing (SLIDRET run from Foxpro session).
*   Now use one common set of coding, so no need to check Glfromstartup for a true/false value.
*
*	October 2001
*	Routine for Foxpro 1705 error - ' File access is denied.' has been modified.   
*	This routine now distinguishes between 3 different situations when this error arises:
*		1. Error occurs on startup (multiple runnings of SLIDRET on same PC)
*		2. Error caused by a query parameter table or internal SLIDRET table
*		3. Error caused by a data table
*
*	January 2002
*	Commented out CASE for error 'Invalid path or file name.'.  It is better to let the
*	OTHERWISE condition handle this.
*	Any time an error condition is raised, output a memory dump of variables active at the time.
*	(LIST MEMORY NOCONSOLE TO FILE cMemDump)
*
*	January 2003
*	The dataset name for the memory dump of variables (see above, January 2002), was 
*	originally 'sysDump.txt'.  This has been modified to include date and time, in the 
*	form 'sysDump.Dyyyymmdd.Thhmmss.txt'.
*	Still left in code to delete old sysDump files using obsolete name format.
*
***********************************************************************************************************

LPARAMETERS cmess, cmess1, cprogname, nerrcode, nlineno

LOCAL ncurrentworkarea		&& Save current work area
LOCAL niconandtype			&& Buttons and icon that appear in Dialogue Box

LOCAL cerrortype			&& E - error, I - information
LOCAL cerrorcode			&& Numeric identifier for message
LOCAL ctitlebartext			&& Header for Dialogue Box
LOCAL cmesstext				&& Message text
LOCAL cMemDump				&& Filename for memory dump
LOCAL cDate					&& Current date

ncurrentworkarea = SELECT(0)
gndispcode = 0						&& Contains the user's reponse to the message box (1 = OK, 2=Cancel)

* Get error code from AERROR, rather than from nerrcode
=AERROR(aErrorArray)
*!*	WAIT 'Error Code=' + LTRIM(STR(aErrorArray(1))) WINDOW

IF gnerror = 0
*!*		gnerror = nerrcode					&& Assign error number to global variable
	gnerror = aErrorArray(1)		&& Assign error number to global variable
*!*	ELSE 
*!*		nerrcode = gnerror
ENDIF

*****************************************************************************************
* Dump all system variables to a text file (sysDump).
* Remove any previous sysDump files before building the new one.
*
* NOTE:  The sysDump file has been renamed to include Date and Time.
*		 However, there may still be sysDump files around using the old naming standard,
*		 so check for this as well.
*
* SYS(5)	- current drive
* SYS(2003)	- current directory
*****************************************************************************************		 
cMemDump = SYS(5) + SYS(2003) + '\sysDump.txt'	&& Still check for obsolete name for sysDump
IF FILE(cMemDump)			&& Remove previous file containing memory dump of variables
	DELETE FILE (cMemDump)
ENDIF

* Remove any old versions of sysDump with new name format.
cMemDump = SYS(5) + SYS(2003) + '\sysDump.D*.T*.txt'
DELETE FILE (cMemDump)	

* New name for sysDump includes Date and Time.
cDate = DTOC(DATE())	&& Convert today's date to character format
* Remove slashes from Date, and colons from Time.
cMemDump = SYS(5) + SYS(2003) + '\sysDump.D' + CHRTRAN(cDate,'/','') + '.T' + ;
		   CHRTRAN(TIME(),':','') + '.txt'

LIST MEMORY NOCONSOLE TO FILE (cMemDump)		&& Dump system variables to a file

*!*	IF Glfromstartup = .T.			&& Faterror is called from startup.prg
	DO CASE
		CASE gnerror = 1569 OR gnerror = 1881
			* Foxpro 1569 error - 'Database "name": File access denied.'
			* Foxpro 1881 error - 'Error loading file - record number...'
*!*				wait 'error 1569 OR 1881 - file access denied' window
			cerrortype = 'E'
			cerrorcode = '00012'
			ctitlebartext = "Error Message / Message d'Erreur "  
			ctitlebartext = ctitlebartext + cerrorcode
			niconandtype = 64       && info sign + OK button
			* For an error like this, tblmess1 cannot be accessed because it is being used by 
			* another application.	So the error message has to be hard-coded here.
*!*				Cmesstext = RTRIM('Another instance of this application is running. Exiting program... / ' ;
*!*							+ chr(13) + 'CONVERT TO FRENCH')
			Cmesstext = RTRIM('Cannot start this program as another version of this program is currently running./ ' ;
						+ chr(13) + 'Ne peut pas lancer ce programme car une autre version de ce programme est prsentement en excution.')
			Gndispcode = MESSAGEBOX(Cmesstext,niconandtype,ctitlebartext)
			SELECT (ncurrentworkarea)	
			ON ERROR							&& Clear the error handler
			
		CASE gnerror = 3 or gnerror = 1705 
			* Foxpro 3 error - ' File is in use.'				
			* Foxpro 1705 error - ' File access is denied.'
*!*				wait 'error 1705 -file access denied' window

*!*				IF EMPTY(gcDataTable) 
			IF LOWER(gcDataTable) = 'startup'	&& Invoked from startup.prg
				* In this case, a system table cannot be accessed because it is being used by 
				* another application.			
				cerrortype = 'E'
				cerrorcode = '00012'
				ctitlebartext = "Error Message / Message d'Erreur "  
				ctitlebartext = ctitlebartext + cerrorcode
				niconandtype =  64       && info sign + OK button
				* For an error like this, tblmess1 cannot be accessed because it is being used 
				* by another application.	So the error message has to be hard-coded here.
*!*	*!*				Cmesstext = RTRIM('Another instance of this application is running. Exiting program... / ' ;
*!*	*!*							+ chr(13) + 'CONVERT TO FRENCH')
				Cmesstext = RTRIM('Cannot start this program as another version of this program is currently running./ ' ;
						+ chr(13) + 'Ne peut pas lancer ce programme car une autre version de ce programme est prsentement en excution.')
			ELSE	
				IF LOWER(gcDataTable) = 'query table/system table'
					* Could not access a non-data table, such as a table belonging to a query,
					* or a system table. 
					DO FORM faterror WITH cmess, cmess1, cprogname, gnerror, nlineno
				ELSE
					* A data table needed to run the query is inaccessible because another 
					* user has a lock on it.  Or there are permission problems.
					SELECT tblmess1 
					LOCATE FOR cmesscode = 'E00013'
					cerrortype = SUBSTR(cmesscode,1,1)
					cerrorcode = SUBSTR(cmesscode,2,5)	
					ctitlebartext = "Error Message / Message d'Erreur "  
					ctitlebartext = ctitlebartext + cerrorcode
					niconandtype =  64       && info sign + OK button
					Cmesstext = RTRIM(Cmesstext&gcLanguage) + ' ' + gcDataTable
				ENDIF
			ENDIF

			Gndispcode = MESSAGEBOX(Cmesstext,niconandtype,ctitlebartext)		
			SELECT (ncurrentworkarea)	
			ON ERROR							&& Clear the error handler
			
		CASE gnerror = 15 OR gnerror = 13		&& If file is not a table
			SELECT tblmess1 
			LOCATE FOR cmesscode = 'I00031'
			cerrortype = SUBSTR(cmesscode,1,1)
			cerrorcode = SUBSTR(cmesscode,2,5)	
			ctitlebartext = "Error Message / Message d'Erreur "  
			ctitlebartext = ctitlebartext + cerrorcode
			niconandtype =  64       && info sign + OK button
			Cmesstext = RTRIM(Cmesstexte) + ' / ' + chr(13) + RTRIM(Cmesstextf)
			Gndispcode = MESSAGEBOX(Cmesstext,niconandtype,ctitlebartext)	
			SELECT (ncurrentworkarea)	
			ON ERROR							&& Clear the error handler
	
		CASE gnerror = 3001 	&& Foxpro does not use this number, traps error if file does not exist
			SELECT tblmess1 
			LOCATE FOR cmesscode = 'I00027'
			cerrortype = SUBSTR(cmesscode,1,1)
			cerrorcode = SUBSTR(cmesscode,2,5)	
			ctitlebartext = "Error Message / Message d'Erreur "  
			ctitlebartext = ctitlebartext + cerrorcode
			niconandtype = 49       			&& stop sign + OK button
			Cmesstext = RTRIM(Cmesstexte) + ' / ' + chr(13) + RTRIM(Cmesstextf)
			Gndispcode = MESSAGEBOX(Cmesstext,niconandtype,ctitlebartext)		
			SELECT (ncurrentworkarea)			
			ON ERROR						&& Clear the error handler
			
		CASE gnerror = 3000		&& Foxpro does not use this number, traps error when another instance of program is running
			SELECT tblmess1 
			LOCATE FOR cmesscode = 'I00032'
			cerrortype = SUBSTR(cmesscode,1,1)
			cerrorcode = SUBSTR(cmesscode,2,5)	
			ctitlebartext = "Error Message / Message d'Erreur "  
			ctitlebartext = ctitlebartext + cerrorcode
			niconandtype = 64       		&& Info sign + OK button
			Cmesstext = RTRIM(Cmesstexte) + ' / ' + chr(13) + RTRIM(Cmesstextf)
			Gndispcode = MESSAGEBOX(Cmesstext,niconandtype,ctitlebartext)		
			SELECT (ncurrentworkarea)
			ON ERROR						&& Clear the error handler
							
*!*			CASE cmess = 'Invalid path or file name.'		
*!*				SELECT tblmess1 
*!*				gnerror = 3002
*!*				LOCATE FOR cmesscode = 'I00028'
*!*				cerrortype = SUBSTR(cmesscode,1,1)
*!*				cerrorcode = SUBSTR(cmesscode,2,5)	
*!*				ctitlebartext = "Error Message / Message d'Erreur "  
*!*				ctitlebartext = ctitlebartext + cerrorcode
*!*				niconandtype = 64       		&& Info sign + OK button

*!*				Cmesstext = RTRIM(Cmesstexte) + ' / ' + chr(13) + RTRIM(Cmesstextf)

*!*				Gndispcode = MESSAGEBOX(Cmesstext,niconandtype,ctitlebartext)		
*!*				SELECT (ncurrentworkarea)	
*!*				ON ERROR						&& Clear the error handler
					
		OTHERWISE
			DO FORM faterror WITH cmess, cmess1, cprogname, gnerror, nlineno
	ENDCASE
*!*	ELSE													&& Faterror is called from main program
*!*		wait 'error outside startup.prg' window
*!*		DO CASE
*!*			CASE nerrcode = 1881		&& Another instance is running and cannot load an object in FRSRetrieval
*!*				ncurrentworkarea = SELECT(0)
*!*				SELECT tblmess1
*!*			
*!*				LOCATE FOR RTRIM(tblmess1.cmesscode) =  'I00032'
*!*	*!*				wait 'message=' + cmesscode window
*!*				cerrortype = SUBSTR(cmesscode,1,1)
*!*				cerrorcode = SUBSTR(cmesscode,2,5)		
*!*					IF Gclanguage = 'E'
*!*					ctitlebartext = 'Error Message '  
*!*				ELSE Gclanguage = 'F'
*!*					ctitlebartext = "Message d'Erreur "
*!*				ENDIF
*!*				ctitlebartext = ctitlebartext + cerrorcode
*!*				niconandtype = 64       && info sign + OK button		
*!*					
*!*				IF Gclanguage = 'E'
*!*					Cmesstext = RTRIM(Cmesstexte) 
*!*	*!*					wait 'message=' + Cmesstext window
*!*				ELSE
*!*				IF Gclanguage = 'F'
*!*					Cmesstext = RTRIM(Cmesstextf)
*!*				ENDIF									
*!*				ENDIF
*!*				Gndispcode = MESSAGEBOX(Cmesstext,niconandtype,ctitlebartext)		
*!*				SELECT (ncurrentworkarea)	
*!*				ON ERROR			&& Clear the error handler
*!*				Gndispcode = 0		&& Clear the error number
*!*				
*!*			CASE gnerror = 15 OR gnerror = 13
*!*				ncurrentworkarea = SELECT(0)
*!*				SELECT tblmess1
*!*				LOCATE FOR RTRIM(tblmess1.cmesscode) =  'I00031'
*!*				wait 'message=' + cmesscode window
*!*				cerrortype = SUBSTR(cmesscode,1,1)
*!*				cerrorcode = SUBSTR(cmesscode,2,5)		
*!*				
*!*				IF Gclanguage = 'E'
*!*					ctitlebartext = 'Error Message '  
*!*				ELSE Gclanguage = 'F'
*!*					ctitlebartext = "Message d'Erreur "
*!*				ENDIF
*!*				
*!*				ctitlebartext = ctitlebartext + cerrorcode
*!*				niconandtype = 64       && info sign + OK button		
			
*!*				IF Gclanguage = 'E'
*!*					Cmesstext = RTRIM(Cmesstexte) 
*!*	*!*					wait 'message=' + Cmesstext window
*!*				ELSE
*!*					IF Gclanguage = 'F'
*!*						Cmesstext = RTRIM(Cmesstextf)
*!*					ENDIF									
*!*				ENDIF
			
*!*				Cmesstext = RTRIM(Cmesstext&gcLanguage) 	&& Replaces above commented-out code	
*!*				Gndispcode = MESSAGEBOX(Cmesstext,niconandtype,ctitlebartext)		
*!*				SELECT (ncurrentworkarea)	
*!*				ON ERROR			&& Clear the error handler
*!*				
*!*			OTHERWISE
*!*				DO FORM faterror WITH cmess, cmess1, cprogname, gnerror, nlineno
*!*		ENDCASE	
*!*	ENDIF

ENDPROC


PROCEDURE CHECKDATADIR
********************************************************************************************************************
*
* PROCEDURE:			Checkdatadir
*
* PROCEDURE LIBRARY:	Retproc.prg
*
* CALLED FROM:			Startup.prg
*						frsRetrieval.tbrRetrieval.cmdDatadir.Click
*						frsRetrieval.Load
*
* CALLS:				retproc.loadpanel
*
* PARAMETERS:			lValiddir:	Logical flag, 'TRUE' indicates all SLIDDBF files are 
*									located in the selected directories.
*						cDir:		SLID data directory
*
* PROCESS: 
* 	This procedure ensures that all SLIDDBF tables are located in the selected directory.
*	This is done by checking all data tables listed in tblTable.
*	If the table is longitudinal, then the subdirectories for all years are checked,
* 	otherwise only the root directory is searched.
*	The listing for SLIDDBF tables is contained in tblTable.
*	tblYear lists only those years for which all SLIDDBF tables exist.
* 
* MAINTENANCE DONE:
*	May 1999
*	If a longitudinal .dbf file is missing, the wrong filename may be displayed.
*	The filename of the missing file contains the year, but instead of displaying
*	the year in question, the maximum year (from tblYear) is shown.
*	To correct this, added an EXIT stmt to break out of the year loop when a .dbf
*	file cannot be found.
*
*	September 1999
*	The year subdirectory for the data tables has been renamed from 'yr9999' to 'year9999'
*	For example, ...\yr1996\xxxxxx.dbf has now become ...\year1996\xxxxxx.dbf
*
*	October 1999
*	Previously, if any datasets were missing, FALSE would be returned to the calling program,
*	and you would be prompted to select another directory.  This has changed so that any years
*	missing longitudinal tables would be marked as unusable (tblYear.lvalid = .F.).
*	As long as the non-longitudinal tables were all present, and at least 1 year had a 
*	complete set of longitudinal tables, a TRUE is returned (no directory prompt).
*	Any years missing data will not be displayed on frmSubtheme.
*
*	May 2002
*	tblYear is now dynamically generated whenever a new GUI session is started or
*	a new SLID data directory is selected.
*	In the past, when a new year of client data became available, the client would have to
*	request that SDD add the new year to the year table.  This meant a new version of the
*	system would have to be distributed with the updated year table.
*	Now, the year table is built automatically by examining the SLID data directory for
*	subdirectories YEARxxxx, where xxxx is the year.  xxxx is stripped off the subdirectory
*	name, and added to tblYear.
*
*   June 2002
*	Added a declaration for array acYears.  When running an executable, program crashed because
*	it complained that acYears could not be found.  It should NOT be necessary to declare
*	acYears because function ADIR() automatically creates it if it does not already exist.
*	Problem did not occur when not running from an executable (i.e. SDD test mode).
*
*	August 2003 (version 2.2)
*	Added call to retproc.loadpanel.  This routine builds new table tblPanel, using the
*	year table.  tblPanel gives users the option of specifying panels instead of years
*	for their longitudinal queries.
*
*	September 20, 2004 (version 2.3)
*	tblTable has been converted to a free table and ISD is now responsible for its maintenance.
*	Since free tables cannot have column names longer than 10 characters, column
*	'Islongitudinal' has been shortened to 'Islong'.
*
*	March 10, 2005	(version 2.3)
*	Have run into the situation where a blank line gets appended to tblTable after a system
*	abend.  Not sure why.  Added check to ignore blank records on tblTable when looking for
*	missing datasets.
*
*********************************************************************************************************************

LPARAMETERS lValiddata, cDir 

LOCAL lMissing			&& SLID data table not found
LOCAL lValiddata  		&& necessary SLID data files are present

*!*	LOCAL cmaxyear			&& The highest year listed in tblYear (used as max value in loop counter)
*!*	LOCAL cminyear			&& The lowest year listed in tblYear (used as min value in loop counter)
LOCAL cSkeleton 		&& Search criteria for function ADIR()
LOCAL cYr				&& Year extracted from subdirectory name
LOCAL cDataloc 			&& directory holding SLID data
LOCAL cTablename		&& SLID data tablename
LOCAL cFilename			&& full pathname of SLID data table

LOCAL nYearcnt			&& Number of records in tblYear
LOCAL nValidYear		&& Number of valid years (no data missing)

DIMENSION acYears(1)	&& Stores subdirectory names YEARxxxx, used for storing annual SLID data.
						&& Used by function ADIR(), which will resize the array automatically.
						
* Initialize variables
cDataloc = cDir 
lMissing = .F.

*!*	SELECT tblYear
*!*	SET FILTER TO
*!*	* Mark all years as valid
*!*	SCAN
*!*		REPLACE lvalid WITH .T.
*!*	ENDSCAN

*********************************************************************************
* NOTE: Following code replaces above commented-out code.
*
* Dynamic loading of Year table is based on available SLID data:
*	- Wipe out old contents of Year table.
*	- Extract year from YEARxxxx subdirectory names in SLID data directory.
*********************************************************************************
SELECT tblYear
SET FILTER TO
ZAP

cSkeleton = RTRIM(cDataloc) + 'YEAR*'
nYearcnt  = ADIR(acYears, cSkeleton, 'D')
=ASORT(acYears,1)		&& Sort on 1st field (subdirectory name YEARxxxx)

IF nYearcnt = 0		&& No year subdirectories found - bad directory
	lMissing = .T.
ELSE				&& Year subdirectories exist
	* Rebuild year table
	SELECT tblYear
	
	FOR j = 1 to nYearcnt
		cYr = SUBSTR(acYears(j,1),5,4)		&& Extract xxxx from YEARxxxx
		APPEND BLANK
		REPLACE tblYear.lchosen with .F. ;
				tblYear.cyear with cYr ;
				tblYear.lvalid with .T.
	ENDFOR
	
*!*		SET TALK OFF			&& Do not display results of CALCULATE
*!*		* Get the highest and lowest year
*!*		CALCULATE MAX(cyear) TO cmaxyear
*!*		CALCULATE MIN(cyear) TO cminyear
*!*		SET TALK ON 

*!*		nmaxyear = VAL(cmaxyear)
*!*		nminyear = VAL(cminyear)
*!*		nYearcnt = nmaxyear - nminyear + 1

	***********************************************************************************
	* Once the year table has been built, we can now verify that SLIDDBF tables are 
	* accounted for in the selected directory.
	***********************************************************************************
	
	* Create array to keep track of those years that are missing longitudinal tables
	DIMENSION alValidYear[nYearcnt]

	* Initialize year array, assuming all years are valid
	FOR j=1 to nYearcnt	
		alValidYear[j] = .T.
	ENDFOR

*!*		Ncurrentworkarea = SELECT()
			
	* Scan through tblTable, which lists needed SLID data tables, and check if all 
	* dbf files are in this directory.
	SELECT 0
	USE tblTable
*!*		GO TOP
*!*		nMaxRec = RECCOUNT()

	* Look for missing tables (non-longitudinal and longitudinal)
*!*		FOR i = 1 TO nMaxRec
	SCAN	&& Scan through tblTable
*!*			lfilenotfound = .F.
		cTablename = RTRIM(tblTable.Tablename)
		
		IF NOT ISBLANK(cTablename)		&& Bypass blank tblTable records
*!*				IF tblTable.islongitudinal = .F.				&& Check for file in cDataloc
			IF tblTable.islong = .F.				&& Check for file in cDataloc
				cFilename = cDataloc + cTablename + '.dbf'			
				IF EMPTY(SYS(2000,cFilename))				&& file not found
					wait "Missing dataset/Fichier absent: " + cFilename window
					lMissing = .T.
					EXIT	&& If even one non-longitudinal table is missing, no sense continuing
				ENDIF
			ELSE			&& Check for data file in the year subdirectories
				j = 1									&& Index into year array
	*!*				FOR iyr = nminyear TO nmaxyear
				SELECT tblYear
				SCAN	&& Scan through tblYear
	*!*					cYr = 'year' + ALLTRIM(STR(iyr))
					cYr = 'YEAR' + RTRIM(tblYear.cyear)
					cFilename = cDataloc + cYr + '\' + cTablename + '.dbf'
					
					IF EMPTY(SYS(2000,cFilename))		&& file not found
						IF alValidYear[j] = .T.			&& Print only 1 'missing' message per year
							wait "Missing dataset/Fichier absent: " + cFilename window
							alValidYear[j] = .F.		&& Year is marked as missing data
							* Mark year as invalid on tblYear
							REPLACE tblYear.lvalid WITH .F.
	*!*							SELECT tblYear
	*!*							LOCATE FOR tblYear.cyear = ALLTRIM(STR(iyr))
	*!*							IF FOUND()
	*!*								REPLACE tblYear.lvalid WITH .F.
	*!*							ENDIF
	*!*							SELECT tblTable
						ENDIF
					ENDIF	
					
					j = j + 1
				ENDSCAN		&& Scanning table tblYear
				
				
				SELECT tblTable		&& Continue searching table tblTable
	*!*				ENDFOR
			ENDIF	&& IF tblTable.islong = .F.
		ENDIF	&& IF NOT ISBLANK(cTablename)
		
*!*			SKIP 1					&& Continue searching table tblTable
*!*		ENDFOR	&& FOR i = 1 TO nMaxRec
	ENDSCAN		&& Scanning table tblTable

	SELECT tblTable				&& Close table
	USE 
ENDIF	&& IF nYearcnt = 0	

IF lMissing = .T.				&& Missing non-longitudinal table - prompt for new directory
	lValiddata  = .F.
ELSE
	nValidYear = ASCAN(alValidYear,.T.)
	IF nValidYear = 0	&& All years are missing data - prompt for new directory
		lValiddata  = .F.
	ELSE
		* Non-longitudinal tables are all present, and at least 1 year with complete data
		lValiddata  = .T.
		* Generate panel table using year table 
		DO loadpanel
	ENDIF
ENDIF

RETURN (lValiddata )

ENDPROC



PROCEDURE LOADPANEL
**********************************************************************************************
*
* PROCEDURE: 	Loadpanel
*
* PURPOSE:		To build the Panel table.
*				
* PROCEDURE LIBRARY: Retproc.prg
*
* INPUT FILES:	tblYear
*
* OUTPUT FILES:	tblPanel
*
* CALLED FROM:	retproc.checkdatadir
*
* MAINTENANCE DONE:
*	July 6, 2004
*	After the panel table is built, added GO TOP.  tblPanel did not always display
*	starting with 1st entry.
*
**********************************************************************************************
LOCAL cMaxyear
LOCAL cPaneltext

LOCAL nMaxyear
LOCAL nMaxpanel
LOCAL nLowyear
LOCAL nHighyear

SELECT tblYear
SET FILTER TO lvalid

SET TALK OFF			&& Do not display results of CALCULATE
* Get the highest valid year
CALCULATE MAX(cyear) TO cMaxyear
SET TALK ON 

nMaxyear = VAL(cMaxyear)

SET FILTER TO

* The number of panels generated depends upon the number of years of available data.
nMaxpanel = INT((nMaxyear - 1993)/3) + 1

SELECT tblPanel		
ZAP				&& Wipe out previous contents

* Generate an entry for each 6-year panel and an entry for each 3-year overlapping panel. 
FOR j=1 to nMaxpanel
	APPEND BLANK
	REPLACE tblPanel.lchosen WITH .F.
	nLowyear  = 1993 + (j-1)*3
	nHighyear = nLowyear + 5
	
	IF nHighyear > nMaxyear
		nHighyear = nMaxyear	&& Do not exceed available years
	ENDIF
	
	* Create a text entry for each 6-year panel. 
	* e.g. for Panel 1, output entry '1 (1993-1998)' 
	cPaneltext = LTRIM(STR(j))         + ' (' + ;
				 LTRIM(STR(nLowyear))  + '-'  + ;
				 LTRIM(STR(nHighyear)) + ')'
	REPLACE tblPanel.cPanel WITH cPaneltext
	
	* Generate an overlapping panel.  
	* This occurs when 2 6-year panels overlap each other 
	* e.g. Panels 1 and 2 overlap on 3 years, so we generate entry '1+2 (1996-1999)' 
	IF j > 1
		APPEND BLANK
		REPLACE tblPanel.lchosen WITH .F.
		i = j - 1
		nHighyear = 1993 + (i-1)*3 + 5	&& Take the highest year from the 1st panel
		
		IF nHighyear > nMaxyear
			nHighyear = nMaxyear	&& Do not exceed available years
		ENDIF
		
		* Take the lowest year from the 2nd panel 
		cPaneltext = LTRIM(STR(i))         + '+'  + ;
					 LTRIM(STR(j))         + ' (' + ;
					 LTRIM(STR(nLowyear))  + '-'  + ;
					 LTRIM(STR(nHighyear)) + ')'
		REPLACE tblPanel.cPanel WITH cPaneltext
	ENDIF
ENDFOR

SELECT tblPanel
GO TOP

ENDPROC



PROCEDURE WRITELINE
********************************************************************************************************************
*
* PROCEDURE: 	Writeline
*
* PURPOSE:		Writes the SQL command lines needed to execute the query to a text file.
*				If the line that is sent through parameter 'CLINE' is more than
*				254 characters, the line is split up into 254 character chunks
*				and written out as such. 
*
* PROCEDURE LIBRARY: Retproc.prg
*
* CALLED FROM:	frsRetrieval.SQLWRITE
*							.sqlwritelong
*
* PARAMETERS:
*   CLINE:		is the line to write to the SQL coded file.
*!*	*	NFILEID:	is used to change the file being written to.
*	CFILENAME:	is the name of the Foxpro table being written to.
*!*	*	CFILENAME:	is used for the function FSIZE that determines the file size.
*	NLNO:		Optional parameter.  Gives the current global line (gl) being worked on.
*
* MAINTENANCE DONE:
*	July 1999
*	Problem with large SQL queries.  All available global lines (gl's) are being used
*	up, and the queries are still not finished.  If this happens, return to calling program.
*	
*	July 1999
*	"IF nlenleft = 0 ... ELSE" changed to
*	"IF nlenleft = 0 ... ENDIF".
*	This allows the current SQL query in parameter 'cline' to be handled by the following code.
*
*	April 2001
*	Commented out useless line of code that used function FCHSHIZE (change size of file).
*	It took the file size of the input file (using FSIZE), and resized the file to the identical
*	size.  In other words, it accomplished nothing.
*
*	December 2001 (Version 2.0)
*	The layouts, labels, and codesets are all written out as 77-byte SAS-like text files.
*	(74 bytes of data, 2 bytes for carriage return/line feed, 1 byte for continuation
*	 character - blank or semi-colon).
*	We have increased the size of codesets from 50 to 250 characters, so the text file
*	will correspondingly be increased to 350 characters.  This will impact all 3 categories.
*	Also, the continuation character will be dropped, because, in SAS, the semi-colon acts
*	as a terminator, not a continuation character!
*
*	November 2002 (Version 2.1)
*	This procedure has been broken into two procedures.  Writeline now is only used to
*	generate SQL code to a text file (.SQL).  All other text files (labels, layouts,
*	codesets, Univariate Statistics) use new procedure Writeline2, which allows you to
*	set the appropriate line length.
*	The parameter Lnosemicolon has been dropped from Writeline.  It served no useful purpose.
*
*	November 2002 (Version 2.1)
*	The text file functions (FSIZE, FCHSIZE, FWRITE) do not work at all for WINDOWS 98 and 
*	WINDOWS ME, although they still work under WINDOWS NT/2000/XP.  
*	As a result, under 98/ME, we are getting gibberish output for the text files.
*	Instead of using these file functions to directly output a text file, we now output
*	to a temporary Foxpro table, then copy the Foxpro table to a text file.
*   The parameter 'nfileid' has been dropped from Writeline.  With the old file functions
*   gone, this parameter is no longer needed.
*
*********************************************************************************************************************

LPARAMETER cline, cfilename, nlno
*!*	LPARAMETER cline, nfileid, cfilename, nlno
*!*	LPARAMETER cline, Lnosemicolon, nfileid, cfilename,nlno

LOCAL clinenotwrote		&& portion of line not yet written out 
LOCAL creturnstr		&& 2-character substring of line, used to check for '..'
LOCAL ctemp2			&& single character from line, used to check for ',' or space
 
LOCAL nbytesnotwrote 	&& number of bytes still to write out
LOCAL nbyteswrote 		&& number of bytes written out
*!*	LOCAL nfilesize			&& size of text file being generated
LOCAL nstartpos			&& start position of character in cline to be subbed
LOCAL nlencline 		&& Number of characters in cline
LOCAL nlenleft			&& number of bytes unused in current gl
LOCAL ninitialpos		&& used when line is too large to fit on existing gl
LOCAL ncurrpos			&& current position in line being parsed
LOCAL noffset

*!*	LOCAL fchstat			&& return code for FCHSIZE (change size) function
*!*	LOCAL fwrstat			&& return code for FWRITE function

LOCAL lmore				&& True indicates there are more 254 chunks to process, used to 
						&& continue or exit DO WHILE LOOP
LOCAL leolcline			&& True indicates end of line in cline has been reached, this flag 
						&& terminates DO WHILE loop

* Record current table before opening a new table		
DO recordworkarea WITH 'nWritelineworkarea','Writeline'

*!*	nfilesize = FSIZE(cfilename)
*!*	WAIT "FSIZE: filesize for file " + cfilename + " is " + STR(nfilesize) window

* What is the purpose of the following line of code?  All it seems to do is reset the file size
* to its original size!  Commented out.
*!*	fchstat=FCHSIZE(nfileid,nfilesize)

* Initialize line parsing variables
clinenotwrote = cline
nbytesnotwrote = LEN(clinenotwrote) && bytes left to write
nbyteswrote = 0 					&& bytes written
nstartpos = 1						&& Initialize starting position in SUBSTR() to 1st character
nlencline = LEN(cline)

nlrecl = 254

clno = LTRIM(STR(nlno))
	
IF gl&clno = ' '
*!*		nlenleft = 254	
	nlenleft = nlrecl
ELSE
*!*		nlenleft = 254 - LEN(gl&clno)   && length left to the max. char length for var. of 254
	nlenleft = nlrecl - LEN(gl&clno)   && length left to the max. char length for var. of 254
ENDIF	
	
* If cline requires more than one GL, then it is necessary to ensure that no GL's start 
* with ".." as this will be misinterpreted by compiler. If a gl is found to begin with ".." 
* then a blank character is inserted into cline, which moves the breakpoint between gl's. 
* This DO WHILE loop modifies cline but doesn't assign characters to any gl's.
	
IF nlenleft < nlencline 			&& An additional Gl is required 
	leolcline = .F.					&& Initialize flag to false
		
	IF nlenleft = 254				&& If the starting gl is empty, then initial start is 
		ninitialpos = 255			&& the 2nd 254 chunk in cline.
	ELSE							&& If the starting gl is not empty, then there is an 
		ninitialpos = nlenleft + 1	&& offset in the 254 chunks.
	ENDIF
		
	ncurrpos = ninitialpos
	
	DO WHILE leolcline = .F.
		creturnstr = SUBSTR(cline,ncurrpos,2)
			
		IF creturnstr = '..'	&& .. indicates symbolic substitution (e.g. X1A5&cy..personid)
			lfound = .F.
			i = 1
			
			* You cannot break a line in the middle of symbolic substitution.
			* Search backwards in cline for a more suitable breakpoint, in particular ',' or ' '	
			DO WHILE NOT lfound		
				ctemp2 = SUBSTR(cline,ncurrpos - i,1)
					
				IF ctemp2 = ' ' OR ctemp2 = ','
					cline = STUFF(cline,1 + ncurrpos - i, 0 , ' ')	&& Insert a blank after 
																	&& current position in cline
					ncurrpos = ninitialpos							&& Reset position in 
																	&& cline to initial position
					lfound = .T.
				ENDIF
					
				i = i + 1	
					
				IF ncurrpos - i = 0		&& This should never occur (reached end of cline 
										&& without finding ',' or ' ')
*!*						wait 'reached end of cline' window
					lfound = .t.		&& This will cause an exit
				ENDIF
			ENDDO
		ELSE
			ncurrpos = ncurrpos + 254 		&&???????
				
			IF ncurrpos >= nlencline 
				leolcline = .T.
			ENDIF
		ENDIF
	ENDDO
ENDIF		&& IF nlenleft < nlencline	
						
* The current SQL line (gl) has reached the maximum of 254 bytes.
* Write the gl out, and prepare a new gl to process the latest SQL query in parameter cline.
IF nlenleft = 0	
*!*		nfilesize = nfilesize + 254	
*!*		fchstat = FCHSIZE(nfileid,nfilesize)
*!*		fwrstat = FWRITE(nfileid,gl&clno)
*!*		WAIT gl&clno window

	* Add new line to Foxpro table
	SELECT(cfilename)
	APPEND BLANK
	REPLACE cText WITH gl&clno
	nlno = nlno + 1
		
	* Do we have enough declared strings (gl's) left to hold the SQL query?
	IF nlno > gnMaxgl
		RETURN
	ENDIF
		
	clno = LTRIM(str(nlno))
*!*		gl&clno = cline			&& This is not a smart thing to do!  What if cline is > 254 bytes?
*!*		nlenleft = 254
	nlenleft = nlrecl
ENDIF
	
IF nlenleft >= nlencline 	&& just add the string
	IF gl&clno = ' '		&& all gl variables are init to spaces, we don't want this space in the SQL
		gl&clno = cline
*!*			wait 'adding to empty gl : ' + LTRIM(STR(NLENcline)) WINDOW
	ELSE
		gl&clno = gl&clno + cline
	ENDIF	
*!*		WAIT gl&clno window
	
ELSE
	IF nlenleft < nlencline		&& If remaining blanks in global var GL are less than length of cline..
		lmore = .T.
		noffset = nlenleft
		
		IF gl&clno = ''			&& If current Gl is empty
			DO WHILE lmore = .T.			
				&&gl&clno = gl&clno + SUBSTR(cline,1,nlenleft) && divide the line in two parts: 1 pt. for the old line and 
				gl&clno = gl&clno + SUBSTR(cline,nstartpos,nlenleft)

*!*					nfilesize = nfilesize + 254	
*!*					fchstat = FCHSIZE(nfileid,nfilesize)
*!*					fwrstat = FWRITE(nfileid,gl&clno)
*!*					WAIT gl&clno window
*!*					wait 'write char to file=' + STR(fwrstat) window
*!*					? gl&clno	

				* Add new line to Foxpro table
				SELECT(cfilename)
				APPEND BLANK
				REPLACE cText WITH gl&clno							
				nlno = nlno + 1										&& Get next Gl variable
				clno = LTRIM(str(nlno))
				
				* Do we have enough declared strings (gl's) left to hold the SQL query?
				IF nlno > gnMaxgl
					RETURN
				ENDIF
				
				IF (nlencline - nstartpos - nlenleft + 1) > 254	&& If remaining chunk is more than 254 characters
					nstartpos = nstartpos + 254 
		*			nlenleft = 254	
					lmore = .T.								&& Repeat DO WHILE LOOP
		*			WAIT gl&clno window
				ELSE										&& If remaining chunk is less than 254 characters
					gl&clno = SUBSTR(cline,nstartpos + 254) && Assign remaining characters to Gl
					lmore = .F.								&& exit DO WHILE LOOP
				ENDIF
			ENDDO		&& DO WHILE lmore = .T.
		
		ELSE			&& Current gl contains characters
			DO WHILE lmore = .T.
				IF nstartpos = 1	&& First time thru add enough char to fill gl
					gl&clno = gl&clno + SUBSTR(cline,nstartpos,nlenleft)
				ELSE		&& Remaining times thru add 254 char to gl because gl will be empty
					gl&clno = gl&clno + SUBSTR(cline,nstartpos,254)
				ENDIF
				
*!*					nfilesize = nfilesize + 254	
*!*					fchstat = FCHSIZE(nfileid,nfilesize)
*!*					fwrstat = FWRITE(nfileid,gl&clno)
*!*					WAIT gl&clno window
				
				* Add new line to Foxpro table
				SELECT(cfilename)
				APPEND BLANK
				REPLACE cText WITH gl&clno
				nlno = nlno + 1								&& Get next Gl variable
				clno = LTRIM(str(nlno))
				
				* Do we have enough declared strings (gl's) left to hold the SQL query?
				IF nlno > gnMaxgl
					RETURN
				ENDIF
				
				IF (nlencline - nstartpos - nlenleft) > 254	&& If remaining chunk is more than 254 characters
*!*						WAIT 'REMAIN CHUNK > 254'WINDOW
					IF nstartpos = 1			&&bug??		&& First time thru add enough char to fill gl
						nstartpos = noffset + 1
					ELSE
						nstartpos = nstartpos + 254
					ENDIF
					
					nlenleft = 254	
					lmore = .T.								&& Repeat DO WHILE LOOP
		*			WAIT gl&clno window
				ELSE						&& If remaining chunk is less than 254 characters
*!*						IF LEN(cline) < 254
*!*						WAIT 'REMAIN CHUNK < 254'WINDOW
					IF nstartpos = 1
						nstartpos = noffset + 1
*!*							WAIT 'REMAIN CHUNK < 254'WINDOW
					ELSE
						nstartpos = nstartpos + 254 
					ENDIF
*!*						nstartpos = nstartpos + 254
					gl&clno = SUBSTR(cline,nstartpos) 		&& Assign remaining characters to Gl
					lmore = .F.								&& exit DO WHILE LOOP
				ENDIF
			ENDDO		&& DO WHILE lmore = .T.		
		ENDIF		&& IF gl&clno = ''
	ENDIF		&& IF nlenleft < nlencline
ENDIF		&& IF nlenleft >= nlencline

* Restore current table		
DO restoreworkarea WITH 'nWritelineworkarea','Writeline'

ENDPROC


PROCEDURE WRITELINE2
********************************************************************************************************************
*
* PROCEDURE: 	Writeline2
*
* PURPOSE:		To write a line to a text file.
*				If the line that is passed through parameter 'CLINE' is longer than
*				the record length in parameter NFILESIZE, the line is split up into 
*				chunks of length NFILESIZE.
*
* NOTE:			Originally, all text files were output by retproc.writeline.
*				However, when the record length was increased from 77 to 350 to prevent
*				truncation of large codesets (.COD), this caused printing problems for 
*				text file .UST (Univariate Statistics).  The extra blanks caused 
*				wrap-around blank lines to be printed out.  For other text files
*				(.LAY, .LAB), this was not a problem because these are never printed out.
*				As a short term fix, writeline2 was developed from a copy of writeline.
*				In writeline2, the code for .SQL was thrown out, and the text length
*				shortened to 96.
*				In the long term:
*					Writeline2 will be modified to accept a record length as a parameter,
*						and all non-SQL text will call it.
*					As well, writeline will be modified to just write out .SQL text.
*				
* PROCEDURE LIBRARY: Retproc.prg
*
* CALLS:		Retproc.recordworkarea
*					   .restoreworkarea
*
* CALLED FROM:	frsRetrieval.univstatchar
*							.univstatdate
*							.univstatmain
*							.univstatnum
*							.writetosasfile 
*							.writecodeset 
*							.writelabels
*
* PARAMETERS:
*   CLINE:			is the line to write to the text file.
*!*	*	NFILEID:		is used to change the file being written to.
*	CFILENAME:		is the name of the Foxpro table being written to.
*!*	*	CFILENAME:		is used for the function FSIZE that determines the file size.
*	NLINESIZE:		specifies the length of the line to be written
*
* MAINTENANCE DONE:
*	July 1999 (Version 1.5)
*	Problem with large SQL queries.  All available global lines (gl's) are being used
*	up, and the queries are still not finished.  If this happens, return to calling program.
*	
*	July 1999 (Version 1.5)
*	"IF nlenleft = 0 ... ELSE" changed to
*	"IF nlenleft = 0 ... ENDIF".
*	This allows the current SQL query in parameter 'CLINE' to be handled by the following code.
*
*	April 2001 (Version 1.8)
*	Commented out useless line of code that used function FCHSHIZE (change size of file).
*	It took the file size of the input file (using FSIZE), and resized the file to the identical
*	size.  In other words, it accomplished nothing.
*
*	December 2001 (Version 2.0)
*	The layouts, labels, and codesets are all written out as 77-byte SAS-like text files.
*	(74 bytes of data, 2 bytes for carriage return/line feed, 1 byte for continuation
*	 character - blank or semi-colon).
*	We have increased the size of codesets from 50 to 250 characters, so the text file
*	will correspondingly be increased to 350 characters.  This will impact all 3 categories.
*	Also, the continuation character will be dropped, because, in SAS, the semi-colon acts
*	as a terminator, not a continuation character!
*
*	July 2002 (Version 2.0)
*	Parameter NLNO (gl line number) has been dropped.  Not needed.
*	Parameter LNOSEMICOLON (continuation character for long lines) has been dropped.
*	Processing of .SQL text dropped.  This will be handled by retproc.writeline.
*	Record length set to 96 for printing purposes.  Currently used to output only
*	Univariate Statistics.
*	
*	September 2002 (Version 2.0)
*	Hard-coded record length of 96 has been replaced by parameter NLINESIZE, which is 
*	input to writeline2 from the calling program.
*	This will allow generation of text files other than just Univariate Statistics, such as
*	codesets, record layouts, labels.
*
*	November 2002 (Version 2.1)
*	The text file functions (FSIZE, FCHSIZE, FPUTS) do not work at all for WINDOWS 98 and 
*	WINDOWS ME, although they still work under WINDOWS NT and WINDOWS XP.  
*	As a result, under 98/ME, we are getting gibberish output for the text files.
*	Instead of using these file functions to directly output a text file, we now output
*	to a temporary Foxpro table, then copy the Foxpro table to a text file.
*	NOTE: By using a temporary Foxpro table in place of a text file, we inadvertantly
*		  introduced a new bug.  We were changing the active table to our temporary
*		  table, which caused problems in the calling program.  To get around this, added
*		  calls to Retproc.recordworkarea and .restoreworkarea to preserve the
*		  current active table in the calling program.
*
*********************************************************************************************************************

LPARAMETER cline, cfilename, nlinesize
*!*	LPARAMETER cline, nfileid, cfilename, nlinesize

LOCAL clinenotwrote 
LOCAL clinetowrite 
LOCAL nbytesnotwrote 
LOCAL nbyteswrote 
LOCAL nposno 
LOCAL nfilesize
LOCAL nstartpos			&& start position of character in cline to be subbed
LOCAL lmore				&& True indicates there are more 254 chunks to process, used to 
						&& continue or exit DO WHILE LOOP
LOCAL nlencline 		&& Number of characters in cline
LOCAL creturnstr 
LOCAL leolcline			&& True indicates end of line in cline has been reached, this flag 
						&& terminates DO WHILE loop

* Record current table before opening a new table		
DO recordworkarea WITH 'nWriteline2workarea','Writeline2'

*!*	nfilesize=FSIZE(cfilename)
*!*	WAIT "FSIZE: filesize for file " + cfilename + " is " + STR(nfilesize) window

* What is the purpose of the following line of code?  All it seems to do is reset the file size
* to its original size!  Commented out.
*!*	fchstat=FCHSIZE(nfileid,nfilesize)

* Initialize text file variables
clinenotwrote = cline
nbytesnotwrote = LEN(clinenotwrote) && bytes left to write
nbyteswrote = 0 					&& bytes written

* Initialize line parsing variables
nstartpos = 1						&& Initialize starting position in SUBSTR() to 1st character
nlencline = LEN(cline)
*!*	nlrecl = 96		&& Length of .UST file has been set to 96 bytes July 2002.  Previously set
*!*					&& at 350, this caused printing problems (wrap-around blanks).
nlrecl = nlinesize	&& Set size of line to be written.
				
* If writing to SAS-like text file
*!*		nlrecl = 74
*!*		nlrecl = 348			&& Length of SAS-like text file increased from 74 to 348 Dec. 2001
*!*		clinenotwrote = cline
*!*		nbytesnotwrote = LEN(clinenotwrote)   && bytes left to write
*!*		nbyteswrote = 0 && bytes written

*!*		nfilesize=FSIZE(cfilename)
*!*	*!*		nfilesize = nfilesize + 255	
*!*		fchstat=FCHSIZE(nfileid,nfilesize)

DO WHILE .T.
	IF nbytesnotwrote = 0
		EXIT
	ENDIF	

	IF nbytesnotwrote > nlrecl
		FOR nbyteswrote = nlrecl TO 1 STEP -1  && Divide the line into two pieces.
*			WAIT 'GREATER THAN 74' WINDOW
*			WAIT clinenotwrote WINDOW	
*			wait 'nbyteswrote=' + STR(nbyteswrote) window
*			wait 'nbytesnotwrote=' + STR(nbytesnotwrote) window
			cbyte = SUBSTR(clinenotwrote,nbyteswrote,1)
*			wait cbyte window

			IF cbyte = ',' OR cbyte = ' '  && you should be able to split the line at a , or ' '
*!*					clinetowrite = PADR(SUBSTR(clinenotwrote,1,nbyteswrote),nlrecl)   && keep sub dividing line into 78 byte chunks
				clinetowrite = SUBSTR(clinenotwrote,1,nbyteswrote)   && subdivide line into chunks
				clinenotwrote = SUBSTR(clinenotwrote,nbyteswrote + 1,nbytesnotwrote)
	*!*				WAIT 'CLINETOWRITE =' + clinetowrite WINDOW
	*!*				WAIT clinenotwrote WINDOW
				nbytesnotwrote = nbytesnotwrote - nbyteswrote
*!*					csemicolon = ';'
				EXIT
			ENDIF
		ENDFOR
	ELSE
		clinetowrite = clinenotwrote
*!*			clinetowrite = PADR(clinenotwrote,nlrecl)
*		WAIT 'CLINETOWRITE =' + clinetowrite WINDOW
	
	* Following code to assign a continuation character has been disabled.
*!*			IF lnosemicolon
*!*				csemicolon = ' '
*!*			ELSE
*!*				csemicolon = ';'
*!*			ENDIF
			
		nbytesnotwrote = 0
	ENDIF
	
	* Add new line to Foxpro table.
	SELECT (cfilename)
	APPEND BLANK
	REPLACE cText WITH clinetowrite	
	
*!*		nfilesize = nfilesize + nlrecl + 3	&& The extra 3 bytes added to the record length is 
*!*											&& for the carriage return + a line feed added by 
*!*											&& function FPUTS (2 characters) + a continuation 
*!*											&& character (a blank or semi-colon).
*!*		nfilesize = nfilesize + nlrecl + 2	&& The extra 2 bytes added to the record length is 
*!*											&& for the carriage return + a line feed added by 
*!*											&& function FPUTS. 
*!*		fchstat = FCHSIZE(nfileid,nfilesize)
	
*!*		WAIT 'FCHSIZE for file ' + str(nfileid) + '. Want to increase size to ' + STR(nfilesize) + '. New file size= ' + STR(fchstat) window	
*!*		WAIT 'FSIZE for expanded file=' + str(FSIZE(cfilename)) window
		
*!*		clinetowrite = clinetowrite + csemicolon
*!*		fputstat = FPUTS(nfileid,clinetowrite)
		
*!*		wait 'FPUTS for file ' + str(nfileid) + '. Number of bytes written=' + str(fputstat) window
*!*		wait clinetowrite window

ENDDO		&& DO WHILE .T.

* Restore current table		
DO restoreworkarea WITH 'nWriteline2workarea','Writeline2'

ENDPROC



PROCEDURE RECORDWORKAREA
**********************************************************************************************
*
* PROCEDURE: 	Recordworkarea
*
* PURPOSE:		To record the work area of the current active Foxpro table.
*				
* PROCEDURE LIBRARY: Retproc.prg
*
* CALLED FROM:	retproc.writeline
*					   .writeline2
*
* PARAMETERS:
* 	cvarcurrentdbf		- the variable used to hold the value of the current dbf
*	cwherefrom			- name of invoking program, used for debugging
*
* NOTES:
* 	1. All of these values get restored in procedure RETPROC.RESTOREWORKAREA.
*	2. This is similar to frsRetrieval.Recordworkarea.  Unfortunately, it is not
*	   possible to call frsRetrieval.Recordworkarea from retproc procedures, so
*	   a similar version was added to retproc.  This is not a desirable solution for 
*	   maintenance reasons, but the only other solution was to convert all calls
*	   to frsRetrieval.Recordworkarea to the retproc version.
*
* MAINTENANCE DONE:
*
**********************************************************************************************

LPARAMETERS cvarcurrentworkarea, cwherefrom

PUBLIC &cvarcurrentworkarea

&cvarcurrentworkarea = SELECT(0)

*!*	wait 'work area = ' + str(&cvarcurrentworkarea) window

ENDPROC



PROCEDURE RESTOREWORKAREA
************************************************************************************************
* PROCEDURE:	frsRetrieval.restoreworkarea
*
* PURPOSE:		To make active the table that was saved in retproc.recordworkarea.
*
* PROCEDURE LIBRARY: Retproc.prg
*
* INPUT:		table to be restored to active status
*
* OUTPUT:		restored table
*
* CALLED FROM:	retproc.writeline
*					   .writeline2
*
* PARAMETERS:
* 	cvarcurrentworkarea	- the variable used to hold the value of the current dbf
* 	cwherefrom			- where this program was called from - used for debugging purposes only
*
* NOTES:
* 	1. All of these values get assigned in procedure RETPROC.RECORDWORKAREA.
*	2. This is similar to frsRetrieval.Restoreworkarea.  Unfortunately, it is not
*	   possible to call frsRetrieval.Restoreworkarea from retproc procedures, so
*	   a similar version was added to retproc.  This is not a desirable solution for 
*	   maintenance reasons, but the only other solution was to convert all calls
*	   to frsRetrieval.Restoreworkarea to the retproc version.
*
* MAINTENANCE DONE:
*
************************************************************************************************

LPARAMETERS cvarcurrentworkarea, cwherefrom

SELECT (&cvarcurrentworkarea)
*!*	wait 'work area 2= ' + str(select(0)) window

RELEASE &cvarcurrentworkarea

ENDPROC



FUNCTION TRIMFILENAME
********************************************************************************************************************
*
* FUNCTION: TRIMFILENAME
*
* PROCEDURE LIBRARY: RETPROC.PRG
*
* CALLED FROM: 
*
* PARAMETERS: 	cfilepathname - Filename which includes path and may or may not include extension
*
* PROCESS:  	Accepts a file name with a path and returns the file name without the path. The file name 
* 				may or may not have an extension.
*
* RETURNS: 		The trimmed version of cfilepathname. If the file name is invalid (ie empty), an empty string is returned.
*
* 
*
*********************************************************************************************************************


LPARAMETER cfilepathname

LOCAL posofperiod 			&& Location of rightmost period in the string 'cfilename'
LOCAL posofslash 			&& Location of rightmost slash in the string 'cfilename'
LOCAL nlenoffname 			&& Number of characters in the string 'cfilename'
LOCAL lvalid 				&& Logical flag True indicates the file name has been successfully trimmed

cfilename = cfilepathname

IF NOT EMPTY(cfilename)		&& valid file name.
	posofperiod	= RAT('.',RTRIM(cfilename))
	IF posofperiod  = 0 				&& The file name does not have an extension
		posofslash = RAT('\',RTRIM(cfilename))
		nlenoffname = LEN(RTRIM(cfilename)) - posofslash    
		cfilename = SUBSTR(cfilename,posofslash + 1,nlenoffname)
		lvalid = .T.
	ELSE
		posofslash = RAT('\',RTRIM(cfilename))
		nlenoffname = LEN(RTRIM(cfilename)) - posofslash - 4   && the -4 is to take off the extension	
		cfilename = SUBSTR(cfilename,posofslash + 1,nlenoffname)
		lvalid = .T.
	ENDIF
ENDIF

IF lvalid = .T.
	RETURN cfilename 
ELSE
	RETURN {}			&& Filename was invalid, return an empty string
ENDIF

ENDFUNC

