***********************************************************************************************
* Module:	Startup
*
* Purpose:	To invoke the SLIDRET system.
*
* Algorithm:
*	This procedure performs 2 checks before starting the program:
* 	1. Prevents the user from opening a second instance of this program. (Error 1569)
* 	2. Ensure that the data directories contains ALL of the sliddbf data files. This also traps
* 	   a common error where the user attempts to access an empty CD drive.
*
*	NOTE: THE GETDIR MESSAGE BOX REQUIRES FRENCH TRANSLATIONS!!!!!!!!!!!!!!
*	Certain public variables have been declared at this point so that they can be accessed from Frsretrieval,
*   retproc.prg and startup.prg
*
* Files:	tblsyscf
*			tblmess1
*			tblyear
*			tblpanel
*
* Calls:	procedure faterror in retprog.prg
*			procedure checkdatadir in retprog.prg
*
* Modifications:
*	March 1999
*	Added logical variable that duplicates the function of frsRetrieval.TestEnvironment.
*	This is needed to determine whether to use production year table tblYearP, or test
*	year table tblYearT.  Not needed in the past because there was only one year table, tblYear.
*	!!!! Change this variable to .f. when preparing a client program. !!!!!
*
*	October 1999
*	2 Year tables no longer needed.  One tblyear can now be used for both production
*	and testing (see above note for March 1999).  All references to tblYearp, tblYeart,
*	and lTestEnvironment commented out.
*
*	May 2000
*	Language preference now available from tblSyscf (field cChosenlang).
*	Text for function GETDIR has been made bilingual.
*
*	October 2000
*	Have removed the QUIT command from tbrRetrieval.cmdQuitsys.Click and placed it here, after
*	the READ EVENTS command.  This was done because we were getting an Application Error
*	when exiting the system via the menu Query/Exit.  The QUIT command was being executed
*	before a CLEAR EVENTS command had been issued.  This meant that Foxpro was being
*	shutdown while the system was still running (event processing not yet finished).
*
*	April 2001
*	Added new global variable 'gcDataTable'.  This is needed by error code 1705
*	(file access denied) and error code 3 (file is in use) in retproc.faterror.  
*	gcDataTable is set to 'startup' here to indicate that errors 3 and 1705 are 
*	being caused by a system table being unavailable due to another instance of SLIDRET 
*	running.
*
*	September 2001
*	The global variable Glfromstartup identifies whether SLIDRET is being run from the
*	startup program (True), or is being run from a Foxpro session (False).
*	Oddly, the value of Glfromstartup is changed to False in the startup program just
*	before the forms are displayed.  This causes problems if the error routine
*	retproc.faterror is called from the forms, causing custom checks to be bypassed.
*	Instead, a standard error message is output in place of a more meaningful error
*	message.  Commented out.
*	(But this just raises another question.  Why do we need 2 sets of error checking
*	 in retproc.faterror?  Why can't we use the same error checks for both startup runs
*	 and Foxpro session runs?  Look into this!)
* 
*   June 2002
*	Added 'SET SAFETY OFF' to prevent system prompts on Startup for 2 situations:
*	1. Deleting sysdump.txt when procedure retproc.faterror is called.
*   2. Zapping tblYear in procedure retproc.checkdatadir.
*   In each of the above cases, the system prompted for permission to proceed, and we
*   don't want any system prompts.
* 
* 	June 2002
*	tblYear now has to be opened for EXCLUSIVE use.  The reason is that tblYear is
*	now zapped and rebuilt in retproc.checkdatadir (called from Startup), and that 
*	requires exclusive control of the table.
*
*	August 2003 (Version 2.2)
*	New table tblPanel is opened and closed here.  tblPanel gives the client the option
*	of specifying panels instead of years for longitudinal queries.  tblPanel is 
*	generated in retproc.checkdatadir, so has to be opened exclusively.
*
*******************************************************************************************

PUBLIC Gndispcode
PUBLIC Glfromstartup
PUBLIC Gnerror
PUBLIC Gclanguage 			&& Indicates the user's language preference ( 'E' = english, 'F' = french)
PUBLIC gcDataTable			&& Used by retproc.faterror for error code 1705 (file access denied)

*!*	LOCAL lTestEnvironment		&& Running in Test environment (T) or production environment (F)
LOCAL lvaliddir
LOCAL cText					&& Dialog text for function GETDIR

* Initialize variables
*!*	lTestEnvironment = .T.
lvaliddir = .T.
Glfromstartup = .T.
Gnerror = 0
gcDataTable = 'startup'		&& Forces retproc.faterror to print message 
							&& 'Another version of this program is running'

SET SAFETY OFF		&& Prevent system prompts when deleting or zapping a table
SET PROCEDURE TO retproc
ON ERROR DO faterror WITH MESSAGE(), MESSAGE(1), PROGRAM(), ERROR()

* Open tables
Ncurrentworkarea = SELECT()
SELECT 0
USE tblsyscf

*!*		wait 'after reopening tblsyscf and before setting gnerror' window
*!*		wait 'gnerror = ' + str(gnerror) window

* If another version of this program is running, then an error must have occurred when 
* attempting to open the systemdbf database. This error generates the foxpro error no. 1569.
* 1705 - File access denied.  Trying to write to a write-protected file.
* 0003 - File is in use.
* 1881 - Problem reading .SCX file
* 2005 - Trying to set a property to an illegal value

* Error message displayed already by faterror for error codes.
IF Gnerror = 1705 OR Gnerror = 1569 OR Gnerror = 3	OR Gnerror = 1881 OR Gnerror = 2005					&&OR gnerror = 13
*!*		wait 'closing program' window
	Gnerror = 0								&& clear error number
	ON ERROR 								&& clear error handling routine
	CLEAR PROGRAM							&& Close program

ELSE										&& This is the only version running, go ahead with data dir error check..
	SELECT 0
	USE tblmess1

*!*		SELECT 0
*!*		IF lTestEnvironment = .T.
*!*			USE tblyeart ALIAS tblyear
*!*		ELSE
*!*			USE tblyearp ALIAS tblyear
*!*		ENDIF

	SELECT 0
	USE tblyear EXCLUSIVE
	
	SELECT 0
	USE tblpanel EXCLUSIVE

	* Retrieve system settings (directory, language)
	SELECT tblsyscf
	cdataloc = RTRIM(tblsyscf.cdatadir)
	Gclanguage = tblsyscf.cChosenlang
	
	* Ensure that all tables are in data directory
	DO checkdatadir WITH lvaliddir, cdataloc

	IF lvaliddir = .F.					&& Repeat loop until user cancels (quits program), or a valid dat dir is selected
		DO WHILE lvaliddir = .F.
			Gnerror = 3001				&& Generate Foxpro error which will cause error routine (faterror)to be called

			DO faterror WITH MESSAGE(), MESSAGE(1), PROGRAM(), ERROR()
			Gnerror = 0					&& Clear error

			DO CASE
			CASE Gndispcode = 1			&& OK, open dialog box and let user search for directory
				IF Gclanguage = 'E'
					cText = 'Where is your data?'
				ELSE
					cText = 'O sont vos donnes?'
				ENDIF
				
				cdataloc = GETDIR(tblsyscf.cdatadir,cText)
				
				IF NOT EMPTY(cdataloc)
					SELECT tblsyscf
					REPLACE tblsyscf.cdatadir WITH RTRIM(cdataloc)
					DO checkdatadir WITH lvaliddir, cdataloc
				ENDIF

			CASE Gndispcode = 2			&& User selected Cancel, quit program
				SELECT tblsyscf
				USE						&& Close table
				QUIT					&& Terminate program
			ENDCASE

		ENDDO			&&{DO WHILE lvaliddir = .F.}
	ENDIF			&&{IF lvaliddir = .F.}


* Close tables, they will be reopened from main program. If these tables are not closed here, 
* then they would need to be closed when exiting program. Either option is feasible.
	SELECT tblsyscf
	USE
	
	SELECT tblmess1
	USE
	
	SELECT tblyear
	USE
	
	SELECT tblpanel
	USE

*!*		Glfromstartup = .F.		&& Why would you do this?  Doesn't make sense!
*!*		wait 'loading DATARET7' window

	DO FORM DATARET7			&& Start program

* Remove comment from line below in compiled version.
* For SLID clients, the following line must not be commented out.
* But for SDD, the Read Events must be commented out.
	READ EVENTS					&& Needed in the compiled version to trigger reading of events
	QUIT						&& Shut down Foxpro
ENDIF

**************************************************************************************************************************


